@@ -16,7 +16,6 @@
 
 from datetime import datetime, timedelta, timezone
 import json
-import logging
 from typing import List, Tuple, Any
 
 import certifi
@@ -71,7 +70,6 @@ class Municipality(Municipality):
         return f"Municipality({self.id}, {self.name}"
 
 
-# TODO: Improve error handling when Eskom responds with an error.
 class Eskom(Provider):
     def __init__(self):
         self.base_url = "https://loadshedding.eskom.co.za/LoadShedding"
@@ -99,13 +97,82 @@ class Eskom(Provider):
             "9": Stage.STAGE_8,
         }.get(status, Stage.UNKNOWN)
 
+    @classmethod
+    def call(cls, url: str) -> Any:
+        try:
+            retries = urllib3.Retry()
+            with urllib3.PoolManager(retries=retries, ca_certs=certifi.where()) as conn:
+                r = conn.request('GET', url)
+                if r.status != 200:
+                    raise urllib3.response.HTTPError(r.status)
+                return r.data
+        except urllib3.exceptions.HTTPError as e:
+            raise ProviderError("Eskom is unavailable.") from e
+
+    @classmethod
+    def parse_schedule_data(cls, data: str) -> List:
+        try:
+            schedule = []
+            soup = BeautifulSoup(data, "html.parser")
+            days_soup = soup.find_all("div", attrs={"class": "scheduleDay"})
+
+            sast = timezone(timedelta(hours=+2), 'SAST')
+            utc = timezone.utc
+            now = datetime.now(sast)
+            for day in days_soup:
+                date_soup = day.find("div", attrs={"class": "dayMonth"})
+                date_str = date_soup.get_text().strip()
+                date = datetime.strptime(date_str, "%a, %d %b")
+                date = date.replace(year=now.year)
+
+                time_soup = day.find_all("a")
+                for time_tag in time_soup:
+                    start_str, end_str = time_tag.get_text().strip().split(" - ")
+                    start = datetime.strptime(start_str, "%H:%M")
+                    end = datetime.strptime(end_str, "%H:%M")
+                    schedule.append((
+                        now.replace(month=date.month, day=date.day, hour=start.hour, minute=start.minute, second=0,
+                                    microsecond=0, tzinfo=sast).astimezone(utc).isoformat(),
+                        now.replace(month=date.month, day=date.day, hour=end.hour, minute=end.minute, second=0,
+                                    microsecond=0, tzinfo=sast).astimezone(utc).isoformat()
+                    ))
+        except Exception as e:
+            raise ProviderError(f"Unable to parse schedule data.") from e
+        else:
+            return schedule
+
+    @classmethod
+    def parse_area_info_data(cls, data: str) -> AreaInfo:
+        try:
+            soup = BeautifulSoup(data, "html.parser")
+            items = soup.find_all("div", attrs={"class": "areaInfoItem"})
+            area_info = {
+                "Province": {
+                    "Id": int(items[0].find("input", attrs={"id": "provinceId"}).get('value').strip()),
+                    "Name": items[0].find("input", attrs={"id": "province"}).get('value').strip(),
+                },
+                "Municipality": {
+                    "Id": int(items[1].find("input", attrs={"id": "municipalityId"}).get('value').strip()),
+                    "Name": items[1].find("input", attrs={"id": "municipality"}).get('value').strip(),
+                },
+                "Suburb": {
+                    "Id": int(items[2].find("input", attrs={"id": "suburbId"}).get('value').strip()),
+                    "Name": items[2].find("input", attrs={"id": "suburbName"}).get('value').strip(),
+                },
+                "Period": items[3].contents[2].strip().split("\xa0to\xa0"),
+            }
+        except Exception as e:
+            raise ProviderError(f"Unable to parse area info data.") from e
+        else:
+            return AreaInfo(**area_info)
+
     def find_suburbs(self, search_text: str, max_results: int = 10) -> List[Suburb]:
         url = "{base_url}/FindSuburbs?searchText={search_text}&maxResults={max_results}".format(
             base_url=self.base_url,
             search_text=search_text,
             max_results=max_results,
         )
-        data = _get_response(url)
+        data = Eskom.call(url)
         return json.loads(data, object_hook=lambda d: Suburb(**d))
 
     def get_municipalities(self, province: Province) -> List[Municipality]:
@@ -113,96 +180,30 @@ class Eskom(Provider):
             base_url=self.base_url,
             province=province.value,
         )
-        data = _get_response(url)
+        data = Eskom.call(url)
         return json.loads(data, object_hook=lambda d: Municipality(**d))
 
     def get_schedule(self, province: Province, suburb: Suburb, stage: Stage) -> Tuple[tuple]:
-        schedule = []
         url = "{base_url}/GetScheduleM/{suburb_id}/{stage}/{province}/3252".format(
             base_url=self.base_url,
             suburb_id=suburb.id,
             province=province.value,
             stage=stage.value,
         )
-        data = _get_response(url)
-        soup = BeautifulSoup(data, "html.parser")
-        days_soup = soup.find_all("div", attrs={"class": "scheduleDay"})
-
-        if not days_soup:
-            logging.log(logging.ERROR, "Unable to parse schedule. {data}".format(data=data))
-            raise ProviderError(f"No data available (Suburb ID: {suburb.id})") from None
-
-        sast = timezone(timedelta(hours=+2), 'SAST')
-        utc = timezone.utc
-        now = datetime.now(sast)
-        for day in days_soup:
-            date_soup = day.find("div", attrs={"class": "dayMonth"})
-            date_str = date_soup.get_text().strip()
-            date = datetime.strptime(date_str, "%a, %d %b")
-            date = date.replace(year=now.year)
-
-            time_soup = day.find_all("a")
-            for time_tag in time_soup:
-                start_str, end_str = time_tag.get_text().strip().split(" - ")
-                start = datetime.strptime(start_str, "%H:%M")
-                end = datetime.strptime(end_str, "%H:%M")
-                schedule.append((
-                    now.replace(month=date.month, day=date.day, hour=start.hour, minute=start.minute, second=0,
-                                microsecond=0, tzinfo=sast).astimezone(utc).isoformat(),
-                    now.replace(month=date.month, day=date.day, hour=end.hour, minute=end.minute, second=0,
-                                microsecond=0, tzinfo=sast).astimezone(utc).isoformat()
-                ))
-
-        return schedule
+        data = Eskom.call(url)
+        return Eskom.parse_schedule_data(data)
 
     def get_schedule_area_info(self, suburb_id: int) -> AreaInfo:
         url = "{base_url}/GetScheduleAreaInfo/?Id={suburb_id}".format(
             base_url=self.base_url,
             suburb_id=suburb_id,
         )
-        data = _get_response(url)
-
-        soup = BeautifulSoup(data, "html.parser")
-        items = soup.find_all("div", attrs={"class": "areaInfoItem"})
-        area_info = {
-            "Province": {
-                "Id": int(items[0].find("input", attrs={"id": "provinceId"}).get('value').strip()),
-                "Name": items[0].find("input", attrs={"id": "province"}).get('value').strip(),
-            },
-            "Municipality": {
-                "Id": int(items[1].find("input", attrs={"id": "municipalityId"}).get('value').strip()),
-                "Name": items[1].find("input", attrs={"id": "municipality"}).get('value').strip(),
-            },
-            "Suburb": {
-                "Id": int(items[2].find("input", attrs={"id": "suburbId"}).get('value').strip()),
-                "Name": items[2].find("input", attrs={"id": "suburbName"}).get('value').strip(),
-            },
-            "Period": items[3].contents[2].strip().split("\xa0to\xa0"),
-        }
-        return AreaInfo(**area_info)
+        data = Eskom.call(url)
+        return Eskom.parse_area_info_data(data)
 
     def get_stage(self) -> Stage:
-        url = "{base_url}/GetStatus".format(base_url=self.base_url)
-        data = _get_response(url)
+        url = "{base_url}/GetStatus".format(
+            base_url=self.base_url,
+        )
+        data = Eskom.call(url)
         return Eskom.stage_from_status(data.decode("utf-8"))
-
-
-def _get_response(url: str) -> Any:
-    try:
-        logging.log(logging.DEBUG, "GET {url}".format(url=url))
-
-        retries = urllib3.Retry(connect=15, read=5, redirect=0, backoff_factor=1)
-        with urllib3.PoolManager(retries=retries, ca_certs=certifi.where()) as conn:
-            r = conn.request('GET', url)
-
-            logging.log(logging.DEBUG, "Response: {url}".format(url=r.status))
-
-            if r.status != 200:
-                raise urllib3.response.HTTPError(r.status)
-            return r.data
-    except urllib3.exceptions.HTTPError as e:
-        logging.log(logging.ERROR, "Eskom is unavailable. {e}".format(e=e))
-        raise ProviderError("Eskom is unavailable.") from None
-
-
-# https://loadshedding.eskom.co.za/LoadShedding/GetSurburbData/?callback=jQuery191009689061272138111_1650382595946&pageSize=100&pageNum=1&searchTerm=&id=342&_=1650382595947
\ No newline at end of file
